import React, {Component} from 'react'
import { Table, Input, Button, Popconfirm, Form, Icon, InputNumber, Select } from 'antd'
import Utils from '@/utils/utils.js'
import './index.scss'

const EditableContext = React.createContext()

const EditableRow = ({ form, index, ...props }) => (
  <EditableContext.Provider value={form}>
    <tr {...props} />
  </EditableContext.Provider>
)

const EditableFormRow = Form.create()(EditableRow)

class EditableCell extends Component {
  state = {
    editing: false
  }

  toggleEdit = () => {
    const editing = !this.state.editing

    this.setState({ editing }, () => {
      if (editing && this.input && this.input.select) {
        this.input.select()
      } else if (editing && this.input && this.input.focus) {
        this.input.focus()
      }
    })
  }

  save = e => {
    const { record, handleSave } = this.props
    this.form.validateFields((error, values) => {
      handleSave({ ...record, ...values })
      if (error && error[e.currentTarget.id]) {
        return
      }
      this.toggleEdit()
    })
  }

  typeChange = (key, val) => {
    const { record, handleSave } = this.props

    handleSave({ ...record, ...{[key]: val} })

    this.toggleEdit()
  }

  renderCell = form => {
    this.form = form
    const { children, dataIndex, record } = this.props
    const { editing } = this.state

    return editing ? (
      <div>
        {dataIndex === 'Column' || dataIndex === 'Text' ? <Form.Item style={{ margin: 0 }}>
          {form.getFieldDecorator(dataIndex, {
            initialValue: record[dataIndex],
            rules: [
              {
                required: dataIndex === 'Column',
                message: 'NOT NULL.',
              },
            ]
          })(<Input ref={node => (this.input = node)} autoComplete="off" onPressEnter={this.save} onBlur={this.save} />)}
        </Form.Item> : null}
        {dataIndex === 'min' || dataIndex === 'max' ? <Form.Item style={{ margin: 0 }}>
          {form.getFieldDecorator(dataIndex, {
            initialValue: record[dataIndex]
          })(<InputNumber ref={node => (this.input = node)} autoComplete="off" onPressEnter={this.save} onBlur={this.save} />)}
        </Form.Item> : null}
        {dataIndex === 'required' ? <Form.Item style={{ margin: 0 }}>
          {form.getFieldDecorator(dataIndex, {
            initialValue: record[dataIndex] || 'false'
          })(
            <Select
              onChange={(value) => {this.typeChange(dataIndex, value)}}
              onBlur={(value) => {this.typeChange(dataIndex, value)}}
              defaultOpen={true}
            >
              <Select.Option value='false'>否</Select.Option>
              <Select.Option value='true'>是</Select.Option>
            </Select>
          )}
        </Form.Item> : null}
        {dataIndex === 'type' ? <Form.Item style={{ margin: 0 }}>
          {form.getFieldDecorator(dataIndex, {
            initialValue: record[dataIndex] || 'text'
          })(
            <Select
              onChange={(value) => {this.typeChange(dataIndex, value)}}
              onBlur={(value) => {this.typeChange(dataIndex, value)}}
              defaultOpen={true}
            >
              <Select.Option value='text'>文本</Select.Option>
              <Select.Option value='number'>数值</Select.Option>
            </Select>
          )}
        </Form.Item> : null}
      </div>
    ) : (
      <div
        className="editable-cell-value-wrap"
        onClick={this.toggleEdit}
      >
        {children}
      </div>
    )
  }

  render() {
    const {
      editable,
      dataIndex,
      title,
      record,
      index,
      handleSave,
      children,
      ...restProps
    } = this.props
    return (
      <td {...restProps}>
        {editable ? (
          <EditableContext.Consumer style={{padding: 0}}>{this.renderCell}</EditableContext.Consumer>
        ) : (
          children
        )}
      </td>
    )
  }
}

class EditTable extends Component {
  constructor(props) {
    super(props)

    let columns = [
      {
        title: 'Column',
        dataIndex: 'Column',
        width: '15%',
        editable: true
      },
      {
        title: 'Text',
        dataIndex: 'Text',
        width: '18%',
        editable: true
      },
      {
        title: '是否必填',
        dataIndex: 'required',
        width: '12%',
        editable: true,
        render: (text, record) => record.required === 'true' ? '是' : '否'
      },
      {
        title: '类型',
        dataIndex: 'type',
        width: '12%',
        editable: true,
        render: (text, record) => record.type === 'number' ? '数值' : '文本'
      },
      {
        title: '最小值',
        dataIndex: 'min',
        width: '13%',
        editable: true
      },
      {
        title: '最大值',
        dataIndex: 'max',
        width: '13%',
        editable: true
      },
      {
        title: '操作',
        align: 'center',
        dataIndex: 'operation',
        render: (text, record) =>
          this.state.dataSource.length >= 1 ? (
            <div>
              <span className="operation-btn" title={props.dict['header.form.up']} onClick={() => this.handleUpDown(record, 'up')} style={{color: '#1890ff'}}><Icon type="arrow-up" /></span>
              <span className="operation-btn" title={props.dict['header.form.down']} onClick={() => this.handleUpDown(record, 'down')} style={{color: '#ff4d4f'}}><Icon type="arrow-down" /></span>
              <Popconfirm
                title={props.dict['header.form.query.delete']}
                okText={props.dict['header.confirm']}
                cancelText={props.dict['header.cancel']}
                onConfirm={() => this.handleDelete(record.key)
              }>
                <span style={{color: '#1890ff', cursor: 'pointer'}}><Icon type="delete" /></span>
              </Popconfirm>
            </div>
          ) : null,
      }
    ]

    this.state = {
      columns: columns,
      dataSource: props.data
    }
  }

  handleUpDown = (record, direction) => {
    const { dataSource } = this.state
    let index = 0

    let _data = dataSource.filter((item, i) => {
      if (item.key === record.key) {
        index = i
      }

      return item.key !== record.key
    })
    if ((index === 0 && direction === 'up') || (index === dataSource.length - 1 && direction === 'down')) {
      return
    }

    if (direction === 'up') {
      _data.splice(index - 1, 0, record)
    } else {
      _data.splice(index + 1, 0, record)
    }

    this.setState({
      dataSource: _data
    })
  }

  handleDelete = key => {
    const dataSource = [...this.state.dataSource]
    this.setState({ dataSource: dataSource.filter(item => item.key !== key) })
  }

  handleAdd = () => {
    const { dataSource } = this.state
    const newData = {
      key: Utils.getuuid(),
      Column: '',
      Text: ''
    }

    this.setState({
      dataSource: [...dataSource, newData]
    })
  }

  handleSave = row => {
    const newData = [...this.state.dataSource]
    const index = newData.findIndex(item => row.key === item.key)
    const item = newData[index]
    newData.splice(index, 1, {
      ...item,
      ...row
    })
    this.setState({ dataSource: newData })
  }

  render() {
    const { dataSource } = this.state
    const components = {
      body: {
        row: EditableFormRow,
        cell: EditableCell
      }
    }
    const columns = this.state.columns.map(col => {
      if (!col.editable) {
        return col
      }
      return {
        ...col,
        onCell: record => ({
          record,
          editable: col.editable,
          dataIndex: col.dataIndex,
          title: col.title,
          handleSave: this.handleSave,
        })
      }
    })
    return (
      <div className="common-excel-edit-table">
        <Button onClick={this.handleAdd} type="primary" className="add-row">
          添加
        </Button>
        <Table
          components={components}
          rowClassName={() => 'editable-row'}
          bordered
          dataSource={dataSource}
          columns={columns}
          pagination={false}
        />
      </div>
    )
  }
}

export default EditTable